Package: optimalFlow
Type: Package
Title: optimalFlow
Version: 1.24.0
Author: Hristo Inouzhe <hristo.inouzhe@gmail.com>
Maintainer: Hristo Inouzhe <hristo.inouzhe@gmail.com>
Description: Optimal-transport techniques applied to supervised flow cytometry gating.
License: Artistic-2.0
Encoding: UTF-8
Depends: dplyr, optimalFlowData, rlang (>= 0.4.0)
Imports: transport, parallel, Rfast, robustbase, dbscan, randomForest,
        foreach, graphics, doParallel, stats, flowMeans, rgl, ellipse
Suggests: knitr, BiocStyle, rmarkdown, magick
VignetteBuilder: knitr
biocViews: Software, FlowCytometry, Technology
RoxygenNote: 7.1.0
git_url: https://git.bioconductor.org/packages/optimalFlow
git_branch: RELEASE_3_23
git_last_commit: e55f2b3
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:47:49 UTC; biocbuild
