# Generated by roxygen2: do not edit by hand

export(costWasserMatchingEllipse)
export(cytoPlot)
export(cytoPlot3d)
export(cytoPlotDatabase)
export(cytoPlotDatabase3d)
export(estimCovCellGeneral)
export(estimationCellBarycenter)
export(f1Score)
export(f1ScoreVoting)
export(labelTransfer)
export(labelTransferEllipse)
export(optimalFlowClassification)
export(optimalFlowTemplates)
export(qdaClassification)
export(tclustWithInitialization)
export(tclust_H)
export(trimmedKBarycenter)
export(voteLabelTransfer)
export(w2dist)
export(wasserCostFunction)
import(dplyr)
import(ellipse)
import(optimalFlowData)
import(rgl)
import(rlang)
importFrom(Rfast,dista)
importFrom(Rfast,rowMaxs)
importFrom(flowMeans,flowMeans)
importFrom(foreach,"%:%")
importFrom(foreach,"%dopar%")
importFrom(graphics,lines)
importFrom(stats,as.dist)
importFrom(stats,cov)
importFrom(stats,cutree)
importFrom(stats,hclust)
importFrom(stats,mahalanobis)
importFrom(stats,predict)
importFrom(stats,rnorm)
importFrom(stats,runif)
